<?php

namespace Drupal\javali_spreadsheets;

use Google_Client;
use Google_Service_Sheets;

/**
 * Class for the service  javali_sheets.base that enables the creation, configuration and data retrieval of a google sheet
 *
 * @author kallado
 */
class JavaliSheets {

    private $client;
    private $sheet_id;
    private $range;
    private $sheetName;

    /**
     *  JavaliSheets constructor.
     */
    public function __construct() {
        $config = \Drupal::config('javali.google_sheets_config');
        $this->client = $this->getClient();
        $this->sheet_id = $config->get('sheet_id');
        $this->range = $config->get('range');
        $this->sheetName = $config->get('sheet_name');
    }

    /**
     * 
     * @return Google_Client
     * @throws Exception
     */
    public function getClient() {
        $client = new Google_Client();
        $client->setApplicationName('Google Sheets API PHP Quickstart');
        $client->setScopes(Google_Service_Sheets::SPREADSHEETS_READONLY);
        $client->setAuthConfig('credentials.json');
        $client->setAccessType('offline');

        // Load previously authorized credentials from a file.
        $credentialsPath = 'token.json';
        if (file_exists($credentialsPath)) {
            $accessToken = json_decode(file_get_contents($credentialsPath), TRUE);
        }
        else {
            // Request authorization from the user.
            $authUrl = $client->createAuthUrl();
            printf("Open the following link in your browser:\n%s\n", $authUrl);
            print 'Enter verification code: ';
            $authCode = trim(fgets(STDIN));

            // Exchange authorization code for an access token.
            $accessToken = $client->fetchAccessTokenWithAuthCode($authCode);

            // Check to see if there was an error.
            if (array_key_exists('error', $accessToken)) {
                throw new Exception(join(', ', $accessToken));
            }

            // Store the credentials to disk.
            if (!file_exists(dirname($credentialsPath))) {
                mkdir(dirname($credentialsPath), 0700, TRUE);
            }
            file_put_contents($credentialsPath, json_encode($accessToken));
            printf("Credentials saved to %s\n", $credentialsPath);
        }
        $client->setAccessToken($accessToken);

        // Refresh the token if it's expired.
        if ($client->isAccessTokenExpired()) {
            $client->fetchAccessTokenWithRefreshToken($client->getRefreshToken());
            file_put_contents($credentialsPath, json_encode($client->getAccessToken()));
        }
        return $client;
    }

    /**
     * Get the API client and construct the service object.
     * 
     * @return array
     */
    public function getdata() {
        $client = $this->client;
        $service = new Google_Service_Sheets($client);
        $spreadsheetId = $this->sheet_id;
        $name = $this->sheetName;
        $range = $this->range;

        try {
            $sheetInfo = $service->spreadsheets->get($spreadsheetId);
        } catch (\Google_Service_Exception $e) {
            $error = json_decode($e->getMessage(), true);
            drupal_set_message($error['error']['message'], 'error');
            return;
        }

        $allsheet_info = $sheetInfo['sheets'];
        $idCats = array_column($allsheet_info, 'properties');

        // checks if the sheet exists
        if (!$this->myArrayContainsWord($idCats, $name)) {
            return drupal_set_message('Sheet not found', 'error');
        }

        $range_format = $name . '!' . $range;

        try {
            $response = $service->spreadsheets_values->get($spreadsheetId, $range_format);
            return $response;
        } catch (\Google_Service_Exception $e) {
            $error = json_decode($e->getMessage(), true);
            drupal_set_message($error['error']['message'], 'error');
            return;
        }
        
    }

    /**
     * 
     * @param string $id 
     */
    public function setSheetId(string $id) {
        $this->sheet_id = $id;
    }

    /**
     * 
     * @param string $range
     */
    public function setRange(string $range) {
        $this->range = $range;
    }
    
    /**
     * 
     * @param string $name
     */
    
    public function setSheetName(string $name){
        $this->sheetName = $name;
    }

    /**
     *  Checks if myArray contains a word as key
     * 
     * @param array $myArray
     * @param type $word
     * @return boolean
     */
    
    function myArrayContainsWord(array $myArray, $word) {
      foreach ($myArray as $element) {
        if ($element->title == $word) {
            return true;
        }
      }
      return false;
    }
}
